searchData={"items":[{"type":"module","title":"ReqSandbox","doc":"[![CI](https://github.com/mcrumm/req_sandbox/actions/workflows/ci.yml/badge.svg)](https://github.com/mcrumm/req_sandbox/actions/workflows/ci.yml)\n[![Docs](https://img.shields.io/badge/hex.pm-docs-8e7ce6.svg)](https://hexdocs.pm/req_sandbox)\n[![Hex pm](http://img.shields.io/hexpm/v/req_sandbox.svg?style=flat&color=blue)](https://hex.pm/packages/req_sandbox)\n\n[Req][req] plugin for [Phoenix.Ecto.SQL.Sandbox][plug-sandbox].\n\nReqSandbox simplifies making concurrent, transactional requests to a\nPhoenix server. Just before making a request, the sandbox metadata is\napplied via the specified request header. If there is no metadata\navailable, then ReqSandbox creates a new sandbox session and saves\nthe metadata for future requests. This is mostly useful in client\ntest environments to ensure logical isolation between concurrent\ntests.","ref":"ReqSandbox.html"},{"type":"module","title":"Usage - ReqSandbox","doc":"The [Ecto SQL Sandbox Usage Guide](guides/usage.livemd) contains a full demonstration of the sandbox features.\n\n```elixir\nMix.install([\n  {:req, \"~> 0.4.0\"},\n  {:req_sandbox, \"~> 0.2.0\"}\n])\n\nreq = Req.new(base_url: \"http://localhost:4000\" |> ReqSandbox.attach()\n\nReq.post!(req, url: \"/api/posts\", json: %{\"post\" => %{\"msg\" => \"Hello, world!\"}}).body\n#=> %{\"data\" => %{\"id\" => 2, \"msg\" => \"Hello, world!\"}}\n\nReqSandbox.delete!(req)\n#=> \"BeamMetadata (g2gCZAACdjF0AAAAA2QABW93bmVyWGQAInZ2ZXMzM2o1LWxpdmVib29...)\"\n```\n\n[req]: https://github.com/wojtekmach/req\n[plug-sandbox]: https://github.com/phoenixframework/phoenix_ecto","ref":"ReqSandbox.html#module-usage"},{"type":"module","title":"License - ReqSandbox","doc":"MIT License. Copyright (c) 2022 Michael A. Crumm Jr.","ref":"ReqSandbox.html#module-license"},{"type":"function","title":"ReqSandbox.attach/2","doc":"Attaches to a Req request.","ref":"ReqSandbox.html#attach/2"},{"type":"function","title":"Options - ReqSandbox.attach/2","doc":"* `:sandbox_url` - The path to the sandbox. The default value is `\"/sandbox\"`.\n\n  * `:sandbox_header` - The header to put on the request. The default value is `:user_agent`.\n\n  * `:sandbox_header_token` - If provided, the sandbox header value. Otherwise, an existing\n    token from the process will be used. If no token is found, one will be requested from the\n    path at `:sandbox_url`. The default value is `nil`.","ref":"ReqSandbox.html#attach/2-options"},{"type":"function","title":"ReqSandbox.delete!/0","doc":"Deletes the sandbox token from the process dictionary if it exists.","ref":"ReqSandbox.html#delete!/0"},{"type":"function","title":"ReqSandbox.delete!/1","doc":"Deletes the sandbox from the server and the process dictionary, if it exists.","ref":"ReqSandbox.html#delete!/1"},{"type":"function","title":"ReqSandbox.token/0","doc":"Returns the current sandbox token or `nil` if no sandbox exists.","ref":"ReqSandbox.html#token/0"},{"type":"function","title":"Examples - ReqSandbox.token/0","doc":"ReqSandbox.token()\n    #=> \"BeamMetadata (g2gCZAACdjF0AAAAA2QABW93bmVyWGQAInZ2ZXMzM2o1LWxpdmVib29...)\"","ref":"ReqSandbox.html#token/0-examples"},{"type":"extras","title":"CHANGELOG","doc":"# CHANGELOG","ref":"changelog.html"},{"type":"extras","title":"v0.2.0 (2024-09-13) - CHANGELOG","doc":"- Requires Req v0.4 or v0.5.\n\n- Requires Elixir v1.13+.","ref":"changelog.html#v0-2-0-2024-09-13"},{"type":"extras","title":"v0.1.2 (2022-12-22) - CHANGELOG","doc":"- Removes POST body content before making the sandbox request.\n\n- Adds missing `content-length` header on the sandbox request.\n\n- Adds `ReqSandbox.token/0` to fetch the current token if it exists.","ref":"changelog.html#v0-1-2-2022-12-22"},{"type":"extras","title":"v0.1.1 (2022-12-13) - CHANGELOG","doc":"- Fixes a regression when using Req's `:base_url` option.","ref":"changelog.html#v0-1-1-2022-12-13"},{"type":"extras","title":"v0.1.0 (2022-10-25) - CHANGELOG","doc":"Initial release","ref":"changelog.html#v0-1-0-2022-10-25"},{"type":"extras","title":"Ecto SQL Sandbox Usage Guide","doc":"# Ecto SQL Sandbox Usage Guide\n\n```elixir\n~S\"\"\"\n__          __  _                          _\n\\ \\        / / | |                        | |\n \\ \\  /\\  / /__| | ___ ___  _ __ ___   ___| |\n  \\ \\/  \\/ / _ \\ |/ __/ _ \\| '_ ` _ \\ / _ \\ |\n   \\  /\\  /  __/ | (_| (_) | | | | | |  __/_|\n    \\/  \\/ \\___|_|\\___\\___/|_| |_| |_|\\___(_)\n\nWelcome to the sandbox setup guide! This guide contains a complete\nPhoenix application with a basic CRUD to demonstrate the features\nof ReqSandbox.\n\nThe setup is pretty long so this message is here to let you know\nthat by default the guide expects the following:\n\n- You are running a PostgreSQL server locally.\n\n- The db user is \"postgres\" and the password is \"postgres\".\n\n- There exists (or the db user can create) a database\n  named \"req_sandbox_guide\".\n\n- Livebook can run a local Phoenix server on port 5001.\n\nIf you need to adjust any of these assumptions, you need to make\nthe necessary changes in this setup block. Note you can make\nchanges here at any time and then run \"Reconnect and setup\" to\nrestart your Livebook session.\n\nOnce your Livebook session is up and running, follow the steps to\nexamine how the Plug Phoenix.Ecto.SQL.Sandbox provides logical\nisolation to your HTTP requests.\n\nHappy hacking!\n\"\"\"\n\n# Modify these values to suit your environment\npg_host = \"127.0.0.1\"\npg_user = \"postgres\"\npg_pass = \"postgres\"\npg_db = \"req_sandbox_guide\"\nport = 5001\n# END: user values\n\nMix.install([\n  {:req, \"~> 0.5.0\"},\n  {:req_sandbox, \"~> 0.2.0\"},\n  {:bandit, \"~> 1.5\"},\n  {:jason, \"~> 1.0\"},\n  {:phoenix, \"~> 1.7.0\"},\n  {:phoenix_ecto, \"~> 4.0\"},\n  {:ecto_sql, \"~> 3.9\"},\n  {:postgrex, \"~> 0.16\"},\n  {:kino, \"~> 0.7\"}\n])\n\nApplication.put_env(:req_sandbox_guide, Repo,\n  url: \"ecto://#{pg_user}:#{pg_pass}@#{pg_host}/#{pg_db}\",\n  pool: Ecto.Adapters.SQL.Sandbox,\n  ownership_timeout: :timer.hours(24)\n)\n\nApplication.put_env(:req_sandbox_guide, ReqSandboxGuide.Endpoint,\n  adapter: Bandit.PhoenixAdapter,\n  render_errors: [formats: [json: ReqSandboxGuide.ErrorJSON], layout: false],\n  http: [ip: {127, 0, 0, 1}, port: port],\n  server: true,\n  secret_key_base: String.duplicate(\"a\", 64)\n)\n\nApplication.put_env(:phoenix, :json_library, Jason)\n\n# ------ Do not modify anything below this line -------\n\ndefmodule Repo do\n  use Ecto.Repo,\n    otp_app: :req_sandbox_guide,\n    adapter: Ecto.Adapters.Postgres\nend\n\ndefmodule Migration0 do\n  use Ecto.Migration\n\n  def change do\n    create(table(\"posts\")) do\n      add(:msg, :string)\n      timestamps(type: :utc_datetime_usec)\n    end\n  end\nend\n\ndefmodule Post do\n  use Ecto.Schema\n  import Ecto.Changeset\n\n  schema \"posts\" do\n    field(:msg, :string)\n    timestamps(type: :utc_datetime_usec)\n  end\n\n  def changeset(post, attrs) do\n    post\n    |> cast(attrs, [:msg])\n    |> validate_required([:msg])\n  end\nend\n\ndefmodule ReqSandboxGuide do\n  def controller do\n    quote do\n      use Phoenix.Controller,\n        namespace: ReqSandboxGuide,\n        formats: [:json]\n\n      import Plug.Conn\n\n      unquote(verified_routes())\n    end\n  end\n\n  def verified_routes do\n    quote do\n      use Phoenix.VerifiedRoutes,\n        endpoint: ReqSandboxGuide.Endpoint,\n        router: Router\n    end\n  end\n\n  defmacro __using__(which) when is_atom(which) do\n    apply(__MODULE__, which, [])\n  end\nend\n\ndefmodule ReqSandboxGuide.PostController do\n  use ReqSandboxGuide, :controller\n\n  action_fallback(ReqSandboxGuide.FallbackController)\n\n  def index(conn, _params) do\n    posts = Repo.all(Post)\n    render(conn, :index, posts: posts)\n  end\n\n  def create(conn, %{\"post\" => post_params}) do\n    changeset = %Post{} |> Post.changeset(post_params)\n\n    with {:ok, %Post{} = post} <- Repo.insert(changeset) do\n      conn\n      |> put_status(:created)\n      |> put_resp_header(\"location\", ~p\"/posts/#{post}\")\n      |> render(:show, post: post)\n    end\n  end\n\n  def show(conn, %{\"id\" => id}) do\n    post = Repo.get!(Post, id)\n    render(conn, :show, post: post)\n  end\n\n  def delete(conn, %{\"id\" => id}) do\n    post = Repo.get!(Post, id)\n\n    with {:ok, %Post{}} <- Repo.delete(post) do\n      send_resp(conn, :no_content, \"\")\n    end\n  end\nend\n\ndefmodule ReqSandboxGuide.FallbackController do\n  use ReqSandboxGuide, :controller\n\n  # This clause handles errors returned by Ecto's insert/update/delete.\n  def call(conn, {:error, %Ecto.Changeset{} = changeset}) do\n    conn\n    |> put_status(:unprocessable_entity)\n    |> put_view(json: ReqSandboxGuide.ChangesetJSON)\n    |> render(\"error.json\", changeset: changeset)\n  end\n\n  # This clause is an example of how to handle resources that cannot be found.\n  def call(conn, {:error, :not_found}) do\n    conn\n    |> put_status(:not_found)\n    |> put_view(json: ReqSandboxGuide.ErrorJSON)\n    |> render(:\"404\")\n  end\nend\n\ndefmodule ReqSandboxGuide.PostJSON do\n  def index(%{posts: posts}) do\n    %{data: for(post <- posts, do: data(post))}\n  end\n\n  def show(%{post: post}) do\n    %{data: data(post)}\n  end\n\n  defp data(%Post{} = post) do\n    %{\n      id: post.id,\n      msg: post.msg\n    }\n  end\nend\n\ndefmodule ReqSandboxGuide.ChangesetJSON do\n  def translate_errors(changeset) do\n    Ecto.Changeset.traverse_errors(changeset, &translate_error/1)\n  end\n\n  def error(%{changeset: changeset}) do\n    # When encoded, the changeset returns its errors\n    # as a JSON object. So we just pass it forward.\n    %{errors: translate_errors(changeset)}\n  end\n\n  def translate_error({msg, _opts}) do\n    msg\n  end\nend\n\ndefmodule ReqSandboxGuide.ErrorJSON do\n  def render(template, _assigns) do\n    %{errors: %{detail: Phoenix.Controller.status_message_from_template(template)}}\n  end\nend\n\ndefmodule Router do\n  use Phoenix.Router\n\n  pipeline :api do\n    plug(:accepts, [\"json\"])\n  end\n\n  scope \"/\", ReqSandboxGuide do\n    pipe_through(:api)\n\n    get(\"/\", PostController, :index)\n    resources(\"/posts\", PostController, except: [:new, :edit, :update])\n  end\nend\n\ndefmodule ReqSandboxGuide.Endpoint do\n  use Phoenix.Endpoint, otp_app: :req_sandbox_guide\n\n  # In an integration server environment, you want to configure timeout\n  # to something reasonable for your workflows. Automated tests usually\n  # require only a short timeout, but short timeouts can interfere with\n  # manual user testing (and demonstrations like this one!) and produce\n  # unexpected results.\n  plug(Phoenix.Ecto.SQL.Sandbox,\n    at: \"/sandbox\",\n    repo: Repo,\n    timeout: :timer.hours(24)\n  )\n\n  plug(Plug.Parsers,\n    parsers: [:urlencoded, :multipart, :json],\n    pass: [\"*/*\"],\n    json_decoder: Phoenix.json_library()\n  )\n\n  plug(Router)\nend\n\n# Creates the guide's database.\n# It may already exist and that's okay.\n{:ok, pgx} =\n  Postgrex.start_link(\n    hostname: pg_host,\n    username: pg_user,\n    password: pg_pass,\n    database: \"\"\n  )\n\nPostgrex.query(pgx, \"CREATE DATABASE #{pg_db}\", [])\nProcess.unlink(pgx)\nProcess.exit(pgx, :shutdown)\n\n# Configures migrations\n_ = Repo.__adapter__().storage_down(Repo.config())\n:ok = Repo.__adapter__().storage_up(Repo.config())\n\n# Starts the Repo and the Endpoint\n{:ok, _} = Supervisor.start_link([Repo, ReqSandboxGuide.Endpoint], strategy: :one_for_one)\n\n# Runs migrations\nEcto.Migrator.run(Repo, [{0, Migration0}], :up, all: true, log_migrations_sql: :debug)\n\n# Puts the Repo in manual mode\nEcto.Adapters.SQL.Sandbox.mode(Repo, :manual)\n```","ref":"usage.html"},{"type":"extras","title":"Introduction to Ecto SQL Sandboxes - Ecto SQL Sandbox Usage Guide","doc":"Welcome back to the sandbox guide! Once the setup above is complete, we can begin making requests! First let's explore sandboxes from the server's perspective.\n\nThe setup block of this Livebook defines a `Repo` module that acts as our app's data store. It is configured to use the [Ecto.Adapters.SQL.Sandbox](https://hexdocs.pm/ecto_sql/Ecto.Adapters.SQL.Sandbox.html) pool to checkout a `DBConnection`. The pool has been set to `:manual` mode, which requires that each process explicitly checkout a connection before it can be used.\n\nOur sample app defines a simple `Post` schema to demonstrate the sandbox. Try to fetch all of the Posts without first checking out a connection (hint: this raises an error).\n\n```elixir\n# Remember, this raises an error! Keep going :)\nRepo.all(Post)\n```\n\nDid you receive a `DBConnection.OwnershipError` in the last step? Good! That means the sandbox's manual mode is working correctly. You must first checkout a DBConnection from the Sandbox:\n\n```elixir\n:ok = Ecto.Adapters.SQL.Sandbox.checkout(Repo)\n```\n\nOnce you have checked out a connection, you can query the database. Let's try to fetch all the posts again:\n\n```elixir\nRepo.all(Post)\n```\n\nSince the list of posts is empty, now is a good time to insert one:\n\n```elixir\npost = Repo.insert!(%Post{msg: \"hello, world!\"})\n```\n\n...and we can select the post we inserted:\n\n```elixir\nRepo.get!(Post, post.id)\n```\n\n...and afterwards we checkin the connection:\n\n```elixir\n:ok = Ecto.Adapters.SQL.Sandbox.checkin(Repo)\n```\n\nYou have just observed how the Ecto SQL Sandbox pool provides logical isolation for concurrent processes. If you think you might have missed it, go back and re-evaluate all of the steps in this section. You will notice that the list of Posts is always empty.\n\nSandbox sessions are transactional, and when the connection is checked in, the transaction is rolled back. The changes are accessible only within the transaction and they are never actually committed to the database.","ref":"usage.html#introduction-to-ecto-sql-sandboxes"},{"type":"extras","title":"Concurrent HTTP Tests - Ecto SQL Sandbox Usage Guide","doc":"This Livebook is configured with a Phoenix server and a simple API for creating and retrieving posts. You can use the [Req](https://github.com/wojtekmach/req) requests package to access the API.\n\nJust like before, try to fetch some posts, this time as an external client using the API. Just like before, without a sandbox connection, this request will fail:\n\n```elixir\nreq =\n  Req.new(\n    base_url: ReqSandboxGuide.Endpoint.url(),\n    headers: [accept: \"application/json\"],\n    max_retries: 0\n  )\n\n# Remember, I return errors! Keep going :)\nReq.get!(req, url: \"/posts\").body\n```\n\nFor concurrent and transactional tests with external HTTP clients, the plug [Phoenix.Ecto.SQL.Sandbox](https://hexdocs.pm/phoenix_ecto/Phoenix.Ecto.SQL.Sandbox.html) exposes a route on your app's Endpoint where external clients may `POST` to spawn a new sandbox session and `DELETE` to stop an active session. The docs use the example route `\"/sandbox\"`:\n\n<!-- livebook:{\"force_markdown\":true} -->\n\n```elixir\n# lib/my_app_web/endpoint.ex\nplug Phoenix.Ecto.SQL.Sandbox,\n  at: \"/sandbox\",\n  repo: MyApp.Repo,\n  timeout: 15_000 # the default\n```\n\nLink: https://hexdocs.pm/phoenix_ecto/Phoenix.Ecto.SQL.Sandbox.html#module-concurrent-end-to-end-tests-with-external-clients\n\nThis Livebook is configured using the `\"/sandbox\"` route. We can use the Req requests package to create\n\n```elixir\nres = Req.post!(req, url: \"/sandbox\")\nsandbox_token = res.body\n```\n\nNow create one or more posts:\n\n```elixir\nmsg = Kino.Input.text(\"Message\", default: \"Hello, world!\")\n```\n\n```elixir\nresult =\n  Req.post!(req,\n    url: \"/posts\",\n    json: %{post: %{msg: Kino.Input.read(msg)}},\n    user_agent: sandbox_token\n  ).body\n```\n\n...and fetch the last post created:\n\n```elixir\nlast_id = get_in(result, ~w|data id|)\nReq.get!(req, url: \"/posts/#{last_id}\", user_agent: sandbox_token).body[\"data\"]\n```\n\n...or fetch the list:\n\n```elixir\nposts = Req.get!(req, url: \"/posts\", user_agent: sandbox_token).body[\"data\"]\nKino.DataTable.new(posts, name: \"Posts\")\n```\n\nWhen you are done, delete the sandbox session:\n\n```elixir\nReq.delete!(req, url: \"/sandbox\", user_agent: sandbox_token).body\n```\n\n...and that is all! Just remember to manage your sandbox token and include it on all requests, or...","ref":"usage.html#concurrent-http-tests"},{"type":"extras","title":"ReqSandbox to the reqscue! 🚀 - Ecto SQL Sandbox Usage Guide","doc":"ReqSandbox simplifies the process of making sandbox requests to a Phoenix server. By attaching the ReqSandbox plugin to your Req request, the sandbox requirement is handled automatically:\n\n```elixir\nreq = req |> ReqSandbox.attach()\n\nfor n <- 1..3 do\n  Req.post!(req, url: \"/posts\", json: %{post: %{msg: \"Hello, #{n}!\"}})\nend\n|> Enum.map(fn %{body: %{\"data\" => data}} -> data end)\n|> Kino.DataTable.new(name: \"Created Posts\")\n```\n\nReqSandbox even works inside Tasks:\n\n```elixir\nTask.async_stream(1..3, fn _ ->\n  msg = \"Hello, #{DateTime.utc_now()}!\"\n  Req.post!(req, url: \"/posts\", json: %{post: %{msg: msg}})\nend)\n|> Enum.map(fn {_, %{body: %{\"data\" => data}}} -> data end)\n|> Kino.DataTable.new(name: \"Created Posts (Task.async_stream)\")\n```\n\nPerform one last listing:\n\n```elixir\nreq\n|> Req.get!(url: \"/posts\")\n|> Map.fetch!(:body)\n|> Map.fetch!(\"data\")\n|> Kino.DataTable.new(name: \"Posts\")\n```\n\n...and when you are finished, delete the active sandbox session:\n\n```elixir\n_ = ReqSandbox.delete!(req)\n```","ref":"usage.html#reqsandbox-to-the-reqscue"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}